'use strict';

const Transform = require('stream').Transform;
const tools = require('./tools');
const Redis = require('./redis');
const stats = require('./statsd');
const Elasticsearch = require('./elasticsearch');

class Plugin extends Transform {
  constructor(config = {}) {
    super({
      objectMode: true,
      highWaterMark: 10000 * 100
    });
    this.abortOnError = false;
    if (config.redis) {
      this.redis = Redis(config);
    }
    if (config.es) {
      this.es = Elasticsearch(config);
    }
    if (config.statsd_url) {
      this.stats = new stats(config);
    }
  }

  // Business logic to be implemented
  handle(data) { 
    if (!data) return null;
    if (data) return data;
  }

  _transform(broker, encoding, next) {
    try {
      if (broker.isAbort) return next(null, broker);
      let result = this.handle(broker.error, broker.data);

      if (result === null) broker.isAbort = true;
      if (result) broker.data = result;
      next(null, broker);
    } catch (error) {
      this.emit('pluginError', error);
      broker.error = tools.ensurePluginError(error);
      if (this.abortOnError) broker.isAbort = true;

      next(null, broker);
    }
  }
}

module.exports = Plugin;
